
// this :
#include <G4LabTemplateOne/Detector.h>

// Geant4 :
#include <G4Element.hh>
#include <G4Material.hh>
#include <G4Box.hh>
#include <G4Sphere.hh>
#include <G4Tubs.hh>
#include <G4SubtractionSolid.hh>
#include <G4LogicalVolume.hh>
#include <G4PVPlacement.hh>
#include <G4PVReplica.hh>
#include <G4ThreeVector.hh>
#include <G4VisAttributes.hh>
#include <G4Colour.hh>
#include <G4SDManager.hh>

// G4LabTemplateOne :
#include <G4LabTemplateOne/CellSD.h>

G4LabTemplateOne::Detector::Detector() {}                
G4LabTemplateOne::Detector::~Detector() {}

G4VPhysicalVolume* G4LabTemplateOne::Detector::Construct() {        

  // Elements
  G4Element* elN = new G4Element("Nitrogen","N",7,14.01*g/mole);
  G4Element* elO = new G4Element("Oxygen","O",8,16.00*g/mole);

  // Materials 
  G4Material* Air = new G4Material("Air",1.29e-03*g/cm3,2);
  Air->AddElement(elN,.7);
  Air->AddElement(elO,.3);
  new G4Material("Aluminium",13,26.98*g/mole,2.7*g/cm3);
  new G4Material("Iron",26,55.85*g/mole,7.87*g/cm3);
  G4Material* Pb = new G4Material("Lead",82,207.19*g/mole,11.35*g/cm3);

  // World :
  G4LogicalVolume* worldLV = 
    new G4LogicalVolume(new G4Box("",10000*cm,10000*cm,10000*cm),
                        Air,"WorldLV",0,0,0);
  G4VPhysicalVolume* worldPV = 
    new G4PVPlacement(0,G4ThreeVector(),"WorldPV",worldLV,0,false,0);

  // Calorimeter :
  G4LogicalVolume* caloLV  = 
    new G4LogicalVolume(new G4Box("",1500*cm,1500*cm,1000*cm),
                        Air,"CaloLV",0,0,0);
  G4PVPlacement* caloPV = 
   new G4PVPlacement(0,G4ThreeVector(),"CaloPV",caloLV,worldPV,false,0);

  //////////////
  /// Barrel ///
  //////////////
  // A boolean operation :
  // Who delete the matrix and given solids ?
  /*
    G4Tubs* chambTube = 
    new G4Tubs ("ChambTube",70*cm,150*cm,200*cm,0.*deg,360.*deg);
  */
  /* Crash with BooleanProcessor of Geant4 3.0*/

  G4RotationMatrix* rot = new G4RotationMatrix();
  rot->rotateY(90*deg);
  G4SubtractionSolid* barrelTube = 
    new G4SubtractionSolid("BarrelTube",
                           new G4Tubs("",70*cm,150*cm,200*cm,0*deg,360*deg),
                           new G4Tubs("",0*cm,50*cm,400*cm,0*deg,360*deg),
                           rot,G4ThreeVector());
  G4LogicalVolume* barrelLV = 
    new G4LogicalVolume(barrelTube,Pb,"BarrelLV",0,0,0);
  new G4PVPlacement(0,G4ThreeVector(0*cm,0*cm,0*cm),
                    "BarrelPV",barrelLV,caloPV,false,1);

  ////////////////////
  /// Left end cap ///
  ////////////////////
  G4LogicalVolume* leftCapLV = 
    new G4LogicalVolume(new G4Tubs("",20*cm,140*cm,20*cm,0*deg,360*deg),
                        Pb,"LeftCapLV",0,0,0);
  new G4PVPlacement(0,G4ThreeVector(0*cm,0*cm,-225*cm),
                    "LeftCapPV",leftCapLV,caloPV,false,1);
  

  /////////////////////
  /// Right end cap ///
  /////////////////////
  G4LogicalVolume* rightCapLV = 
    new G4LogicalVolume(new G4Box("",140*cm,140*cm,10*cm), //Half sizes.
                        Air,"RightCapLV",0,0,0);
  G4VPhysicalVolume* rightCapPV = 
    new G4PVPlacement(0,G4ThreeVector(0*cm,0*cm,+225*cm),
                      "RightCapPV",rightCapLV,caloPV,false,1);
  
  // Strips :
  G4LogicalVolume* stripLV = 
    new G4LogicalVolume(new G4Box("",7*cm,140*cm,10*cm), //Half sizes.
                        Air,"StripLV",0,0,0);
  G4VPhysicalVolume* stripPV = 
    new G4PVReplica("StripPV",stripLV,rightCapPV,kXAxis,20,14*cm);

  // Cells :
  //G4Box* cellSolid = new G4Box("",7*cm,7*cm,10*cm); //Half sizes
  G4Sphere* cellSolid = 
    new G4Sphere("",0.0*m,7*cm,0.0*deg,360.0*deg,0.0*deg,90.0*deg);
  G4LogicalVolume* cellLV = new G4LogicalVolume(cellSolid,Pb,"CellLV",0,0,0);
  new G4PVReplica("CellPV",cellLV,stripPV,kYAxis,20,14*cm);

  CellSD* cellSD = new CellSD();
  //cellSD->SetVerboseLevel(0);
  G4SDManager::GetSDMpointer()->AddNewDetector(cellSD);
  cellLV->SetSensitiveDetector(cellSD);

  // Visualization attributes
  // Who delete the G4VisAttributes objects ?
  worldLV->SetVisAttributes(G4VisAttributes(false));
  leftCapLV->SetVisAttributes(new G4VisAttributes(G4Colour(1,0,0)));
  barrelLV->SetVisAttributes(new G4VisAttributes(G4Colour(1,1,0)));

  return worldPV;
}
