#ifndef HEPVis_SoTools_h
#define HEPVis_SoTools_h

/*
 * A helper class with some common usefull functions.
 */

#include <Inventor/SbString.h>

#include <HEPVis/SbStyles.h>

class SbVec3f;
class SoNode;
class SoType;
class SoSeparator;
class SoBaseKit;
class SoFullPath;
class SoCamera;
class SoTransform;


class SbVisitor {
public:
  virtual ~SbVisitor(){}
public:
  virtual SbBool visit(SoFullPath&) = 0;
};

class SoTools {
public:
  enum HorizontalJustification {
    LEFT   = 0x00,
    CENTER = 0x01,
    RIGHT  = 0x02
  };
  enum VerticalJustification {
    BOTTOM = 0x00,
    MIDDLE = 0x01,
    TOP    = 0x02
  };

  static SoNode* searchNode(SoNode*,SoType);
  static SoNode* searchNode(SoNode*,const SbString&);
  static SoNode* searchNode(SoNode*,const SbString&,SoType);
  static SbBool isChildOf(SoNode*,SoNode*);

  static void addPolygonToNode(SoSeparator*,
                               int,SbVec3f*,const SbVec3f&,
                               const SbString& = "");
  static void addLinesToNode(SoSeparator*,
                             int,SbVec3f*,
                             const SbString& = "");
  static void addSegmentsToNode(SoSeparator*,
                                int,SbVec3f*,
                                const SbString& = "");
  static void addNurbsCurveToNode(SoSeparator*,
                                   int,SbVec3f*,
                                  const SbString& = "");
  static void addSplineCurveToNode(SoSeparator*,
                                    int,SbVec3f*,
                                   const SbString& = "");
  static void addLagrangePolynomialCurveToNode(SoSeparator*,
                                               int,SbVec3f*,
                                               const SbString& = "");
  static void addPointsToNode(SoSeparator*,
                              int,SbVec3f*,
                              SbMarkerStyle,int,
                              const SbString& = "");
  static void addAlignedStringToNode(SoSeparator*,
                                     const SbString&,
                                     const SbString&,
                                     SbBool,
                                     const SbString&,
                                     const SbVec3f&,
                                     const SbVec3f&,
                                     const SbVec3f&,
                                     float,
                                     HorizontalJustification,
                                     VerticalJustification,
                                     SbBool = FALSE);

  static SbBool setKitPartValue(SoBaseKit*,const SbString&,const SbString&);
  static SbBool getKitPartValue(SoBaseKit*,const SbString&,SbString&);
  static SbBool toggleKitPartValue(SoBaseKit*,const SbString&);
  static void resetHighlight(SoNode*);
  static void highlight(SoFullPath&);

  static void pointAt(SoCamera&,const SbVec3f&,const SbVec3f&);
  static void lookAt(SoCamera&,const SbVec3f&,const SbVec3f&);

  static SbBool write(SoNode& scene_graph,
                      const SbString& file,
                      const SbString& format,
                      SbBool binary,
                      SbBool generateAltRep);

  static SbBool setManip(SoNode*,SoTransform*);
  static void removeManips(SoNode&);
  static void setManips(SoNode&);

  static SbBool visit(SoNode&,SoType,SbVisitor&);

  static SbBool detectIntersection(SoNode&,SbBool stopAtFirst);
};

#endif
