#/////////////////////////////////////////////////////////////////////////////
#/////////////////////////////////////////////////////////////////////////////
#/////////////////////////////////////////////////////////////////////////////
#
# Example of tcl script using a root file.
#
#/////////////////////////////////////////////////////////////////////////////

source $env(ONXLABROOT)/scripts/Tcl/AIDA.tcl

IAnalysisFactory aida -this [AIDA_createAnalysisFactory]

IPlotter plotter -this [aida create_plotter]

# 2x2 regions :
plotter createRegions 2 2 0
plotter setTitle "Analysis Rio example"

# Cleanup memoryTree :
ITree memoryTree -this [aida find_memoryTree]
memoryTree rmdir "/"

# Get an AIDA tree from a ROOT file :
set fileName $env(ONXLABROOT)/examples/data/analysis.root
session destroyManager $fileName

ITree rioTree -this [aida create_tree $fileName root 1 0]
rioTree ls

#/////////////////////////////////////////////////////////////////////////////
# First region :
#  A XPAGE plot with two histograms.  
#!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!
rioTree cd histograms
rioTree ls

# Get some histograms from the file :
IHistogram1D gauss -this [rioTree find_histogram1D gauss]
IHistogram1D BW -this [rioTree find_histogram1D BW]
#puts [gauss mean]

# Plot :
IPlotterRegion region -this [plotter currentRegion]

IPlotterStyle plotterStyle -this [region style]
IAxisStyle xAxisStyle -this [plotterStyle xAxisStyle]
ITextStyle xTextStyle -this [xAxisStyle tickLabelStyle]
xTextStyle setColor red

IAxisStyle yAxisStyle -this [plotterStyle yAxisStyle]
ITextStyle yTextStyle -this [yAxisStyle tickLabelStyle]
yTextStyle setColor red
yTextStyle setFont "TTF/couri"

#FIXME : Have to do the below by using the AIDA styles :
region setParameter "plotter.binsStyle\[0\]\.color" "1 0 0"
region setParameter "plotter.binsStyle\[0\]\.modeling" "wire_boxes"
region setParameter "plotter.binsStyle\[1\]\.color" "0 1 0"
region setParameter "plotter.binsStyle\[1\]\.modeling" "boxes"
region setParameter infosRegionVisible TRUE

region plot [BW cget -this]
region plot [gauss cget -this]

rename BW ""
rename gauss ""
rename region ""

#/////////////////////////////////////////////////////////////////////////////
# Second region :
#  A XY plot.  
#!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!
plotter next

IHistogram2D gauss_BW -this [rioTree find_histogram2D gauss_BW]

IPlotterRegion region -this [plotter currentRegion]
region setParameter infosRegionVisible FALSE

region plot [gauss_BW cget -this]

rename gauss_BW ""
rename region ""

#/////////////////////////////////////////////////////////////////////////////
# Third region :
#  Get a tuple, create an histo from one of its column, plot it. 
#!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!
plotter next
rioTree cd ..
rioTree cd tuples
rioTree ls

ITuple tuple1 -this [rioTree find_tuple tuple1]

IHistogram1D t1_v2 -this [aida create_histogram1D memoryTree t1_v2 "tuple1/v2" 10 0. 10.]

IEvaluator eval1 -this [aida create_evaluator rioTree v2]
IFilter filter1 -this [aida create_filter rioTree ""]

tuple1 project [t1_v2 cget -this] [eval1 cget -this] [filter1 cget -this]

IPlotterRegion region -this [plotter currentRegion]
region setParameter infosRegionVisible FALSE
region plot [t1_v2 cget -this]

# -acquire gives ownership of the C++ object to the Tcl command.
eval1 -acquire
rename eval1 ""

filter1 -acquire
rename filter1 ""

rename t1_v2 ""
rename tuple1 ""
rename region ""
#/////////////////////////////////////////////////////////////////////////////
# Fourth region :x1
#  Get a tuple, create an histo from one of its column, plot it. 
#!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!
plotter next

ITuple tuple2 -this [rioTree find_tuple tuple2]

IHistogram1D t2_gauss -this [aida create_histogram1D memoryTree t2_gauss "tuple2/rand_gauss" 100 -5 5]

IEvaluator eval2 -this [aida create_evaluator rioTree rand_gauss]

IFilter filter2 -this [aida create_filter rioTree "(fabs(rand_gauss)<=2.)&&(step>=1.)"]

tuple2 project [t2_gauss cget -this] [eval2 cget -this] [filter2 cget -this]

IPlotterRegion region -this [plotter currentRegion]
region plot [t2_gauss cget -this]

rename tuple2 ""

eval2 -acquire
rename eval2 ""

filter2 -acquire
rename filter2 ""

#/////////////////////////////////////////////////////////////////////////////
# Fitting :
#/////////////////////////////////////////////////////////////////////////////
IFitFactory ff -this [aida createFitFactory]
IFitter fitter -this [ff createFitter "" "" ""]
IFitResult fitResult -this [fitter fit [t2_gauss cget -this] "G"]
IFunction fitGauss -this [fitResult fittedFunction]

fitResult -acquire
rename fitResult ""

# -acquire gives ownership of the C++ object to the Tcl command.
fitter -acquire
rename fitter ""

ff -acquire
rename ff ""

region plot [fitGauss cget -this]

rename t2_gauss ""
rename fitGauss ""
rename rioTree ""
rename region ""

plotter -acquire
rename plotter ""

